/*
 *	   o p e n p r o g_ c
 *
 *	This file is part of the LIBRLL library.
 *	It handles opening a RLL based program.
 *
 *	For a program file we (potentially) have to find the
 *	start of the BSS area by searching, although as a
 *	small optimisation we assume that the standard C start-up
 *	module starts with a branch to the start code and that the
 *	address of the BSS area is in the 4 bytes before the
 *	target of the branch.
 *
 * (c) Copyright 1999 David J. Walker
 *	   Email:  dave.walker@icl.com
 *
 *	   Permission to copy and/or distribute is granted under the
 *	   following conditions:
 *
 *	   1). This notice must remain intact.
 *	   2). The author is not responsible for the consequences of use
 *		   this software, no matter how awful, even if they arise
 *		   from defects in it.
 *	   3). Altered versions must not be represented as being the
 *		   original software.
 */

#include "librll.h"
#include <fcntl.h>


RLL_FILE *	RLL_OpenProg (char * filename)
{
	int 	fd;

	if (0 <= (fd = open(filename, O_RDONLY)))
	{
		return NULL;
	}
	return RLL_OpenProgFile(fd);
}


RLL_FILE * RLL_OpenProgFile (int fd)
{
	RLL_FILE * rp;
	long	   pos;

	/*
	 *	Allocate standard areas.
	 */
	if (NULL == (rp = _RLL_OpenMem(fd)))
	{
		return NULL;
	}

	/*
	 *	Failed to do optimised find of BSS area
	 *	so we find it the hard way by searching
	 *	the binary of the program.
	 */
	for (pos=0; ; pos += 2)
	{
		char	bufw[2];

		(void) lseek (rp->fd, pos, SEEK_SET);
		if (sizeof(bufw) != read (rp->fd, bufw, sizeof(bufw)))
		{
			(void) RLL_Close (&rp);
			return NULL;
		}
		if (strncmp (bufw, "<<", sizeof(bufw)))
		{
			continue;
		}
		if (RLL_ERROR_NONE == _RLL_BssArea(rp, pos))
		{
			break;
		}
	}
	return rp;
}
